/* Put in the public domain 2000 by Sam Trenholme */
/* Kiwi Main:  The part of the Kiwi rouines with main() in it and the
   relevent command line parsers */

#include "JessStr/JessStr.h"
#include <stdio.h>
#include <sys/types.h>

/* Function prototypes */
void gencookie();

/* Softerror: Exit with an error code allowing the MTA to try again */
int softerror(char *errormsg) {
    fprintf(stderr,"Soft error: %s\n",errormsg);
    exit(111); /* Exit code 111: Try again later */
    }

/* Harderror: Exit with an error code NOT allowing the MTA to try again */
/* As a general rule, use softerror when manipulations on non-user-controlled
   data go haywire, and use harderror when manipulations of user-controlled
   data go haywire */
int harderror(char *errormsg) {
    fprintf(stderr,"Fatal error: %s\n",errormsg);
    exit(100); /* Exit code 100: Return to sender */
    }

int main(int argc, char **argv) {
    js_string *arg, *tstr, *tstr2, *kiwirc;
    char badstr[256];
    int shift = 0;
    int tnum = 0;

    /* Initialize js_string objects */
    if((arg = js_create(256,1)) == 0) 
        softerror("Unable to allocate arg.  Try later.");
    js_set_encode(arg,JS_US_ASCII);
    if((tstr = js_create(256,1)) == 0) 
        softerror("Unable to allocate tstr.  Try later.");
    js_set_encode(tstr,JS_US_ASCII);
    if((tstr2 = js_create(256,1)) == 0) 
        softerror("Unable to allocate tstr2.  Try later.");
    js_set_encode(tstr2,JS_US_ASCII);
    if((kiwirc = js_create(256,1)) == 0) 
        softerror("Unable to allocate kiwirc.  Try later.");
    js_set_encode(kiwirc,JS_US_ASCII);
    
    /* Regardless of anything else, we do need to read the user's .kiwirc
       file */
    if(find_kiwirc(kiwirc) == JS_ERROR)
        softerror("Error trying to locate .kiwirc file.");
    /* To do: Make sure the .kiwirc file has sane perms (600 or 700) */
    if(read_kiwirc(kiwirc,tstr,&tnum) == JS_ERROR)
        softerror("Error in reading .kiwirc file");
    if(tnum != 0) { /* If there was a syntax error in the .kiwirc file */
        if(tnum < -9000 || tnum > 9000)
	    softerror("Error number is out of bounds reading .kiwirc");
        sprintf(badstr,"%d",tnum); /* I really need a js_itoa function */
        js_qstr2js(tstr2,"Syntax error in .kiwirc file: ");
        js_append(tstr,tstr2);
        js_qstr2js(tstr,", line ");
        js_append(tstr,tstr2);
        js_qstr2js(tstr,badstr);
        js_append(tstr,tstr2);
        js_js2str(tstr2,badstr,100);
        softerror(badstr);
        } 

    /* Look at argv[0] and act accordingly */
    if(argv[0] != 0) {
        if(js_qstr2js(arg,argv[0]) == JS_ERROR) 
            harderror("Can not parse argv[0].");
        }
    else 
        harderror("argv[0] is a NULL pointer!");

    /* Now then, let's see how we should run Kiwi */

    if(js_qstr2js(tstr,"c") == JS_ERROR) 
        softerror("Can not give tstr a value, strange.");

    tnum = js_match(tstr,arg);

    if(tnum == JS_ERROR)
        harderror("Unable to run match on argv[0].");
    if(tnum == 0 || tnum == 2) /* Command-line cookie generation */ {
        gencookie();	
	exit(0);
        } 

    harderror("Invalid argv[0] value.");

    }

/* Generate a short, mid, and long timeout Kiwi cookies, and output them to
   the standard output.
   input: none
   output: none
   global vars used: But only in kead_kvar that this calls
*/
void gencookie() {
    /* Set up the key for encrypting data */
    js_string *tstr, *key;
    u_int32_t data;
    if((tstr = js_create(256,1)) == 0)
        softerror("Can not allocate temp string in gencookie");
    js_set_encode(tstr,JS_US_ASCII);
    if((key = js_create(256,1)) == 0)
        softerror("Can not allocate key string in gencookie");
    js_set_encode(key,JS_US_ASCII);

    js_qstr2js(tstr,"kiwi_key");
    read_kvar(tstr,key);

    kjs_key_setup(key);
    data = 0xA7001100; /* CODE HERE: make this time shifted */
    kjs_make_cookie(data,tstr);
    js_show_stdout(tstr);
    printf("\n");
    return;
    }

